/**
 * @file Click_Pollution_WSP2110.c
 * @brief <h2> Pollution click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * Pollution click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        15 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    FT900
 * -<b> Dev. Board    </b> :    EasyFT90x v7
 * -<b> Ext. Modules  </b> :    Pollution click board, EasyTFT
 * -<b> SW            </b> :    mikroC PRO for FT90 v2.0.0
 *
 */

/**
  * @page Dev. board setup
  *
  * 1) Put Pollution click in mikroBUS 1 slot.
  * 2) Button press level for PORT 55-66 I/O group set to GND.
  * 3) Set pull-up for GP64.
  */

/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>
#include <math.h>


#include "pollution.h"
#include "pollution_gui.h"

/******************************************************************************
* Preprocessors
*******************************************************************************/
#define CHANNEL 4

#define MAX_R_LOAD 9950.0

/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
sbit MIKROBUS1_RST  at GPIO_PIN1_bit;
sbit TAST_GP64      at GPIO_PIN64_bit;

float v_load, v_load_old = 0;
char str_buffer[16];

float r_0;
float r_s;
float r_l, r_l_old = 0;

float ratio, ratio_old = 0;
float ppm, ppm_old = 0;

uint8_t button_state_gp64 = 0xFF;

uint8_t refresh_counter = 0;

char inequality_sign = ' ';

/******************************************************************************
* Function Prototypes
*******************************************************************************/
static void system_init( void );
static void port_init( void );

static void calibration( void );
static void adjust_load();

uint8_t button_pressed(uint8_t state, uint8_t* old_state);

/******************************************************************************
* Private Functions
*******************************************************************************/

/**
 *  System initialisation.
 */
static void system_init( void )
{
    port_init();

    gui_init();
}

/**
 *  MCU's port initialisation.
 */
static void port_init( void )
{
    GPIO_Digital_Output(&GPIO_PORT_00_07, _GPIO_PINMASK_1);

    GPIO_Digital_Input(&GPIO_PORT_64_66, _GPIO_PINMASK_0);
    MIKROBUS1_RST = 1;
}

/**
 *  Procedure for sensor calibration.
 */
void calibration()
{
    gui_print_calibration_page();

    while( ! button_pressed(TAST_GP64, &button_state_gp64) )
    {
        // Refresh V load value on display every 250ms.
        if(! (refresh_counter++ % 10))
            v_load_old = gui_refresh_value(measure_v_load(), v_load_old, 235, 80, "%.3f");
        Delay_ms(25);
    }
    // Save sensor resistance in normal condition with MAX R load.
    r_0 = sensor_resistance(measure_v_load(), MAX_R_LOAD);

    gui_clear_page();
}

/**
 *  Procedure for adjusting load.
 */
void adjust_load()
{
    gui_print_adjust_load_page();
    
    while( ! button_pressed(TAST_GP64, &button_state_gp64) )
    {
        // Refresh value on display every 250ms.
        if(!(refresh_counter++ % 10)) 
        {
            r_l = load_resistance(measure_v_load(), r_0);
            r_l_old = gui_refresh_value(r_l / 1000, r_l_old, 115, 80, "%.2f K[ohm]");
        }        
        Delay_ms(25);
    }
    // Save adjusted load resistance.
    r_l = load_resistance(measure_v_load(), r_0);

    gui_clear_page();
}

/**
 *  Function for detecting if button is pressed, with polling method.
 *  \param state Button state.
 *  \param old_state Latest detected state.
 *  \return Function returns 0xFF if falling edge is detected, 0x00 otherwise.
 */
uint8_t button_pressed(uint8_t state, uint8_t* old_state)
{
    if(!state && *old_state)
    {
        *old_state = state;
        return 0xFF;
    }

    *old_state = state;
    return 0x00;
}

/******************************************************************************
* Public Functions
*******************************************************************************/
void main()
{
    system_init();

    calibration();
    adjust_load();

    gui_print_measure_page();
    while(1)
    {
        r_s = sensor_resistance(measure_v_load(), r_l);
        
        ratio = r_s / r_0 > 1 ? 1.0 : r_s / r_0;
        ratio_old = gui_refresh_value(ratio, ratio_old, 235, 100, "%.2f");

        ppm = ratio_to_ppm(ratio);
        ppm_old = gui_refresh_value(ppm, ppm_old, 235, 160, "%.1f");

        gui_update_inequality_sign(ppm);
        
        Delay_ms(250);
    }
}
/*************** END OF FUNCTIONS *********************************************/